@@ -210,6 +210,8 @@ int local_exit(int status) {
  *  14-Sep-1994      Marty Stanton       Brandeis University
  *
  */
+/* Commented out 19 June 2008 because of optimization bug in gcc
+   -- HJB
 int getbo()
 {
   long i4;
@@ -227,6 +229,7 @@ int getbo()
   else
     return(2);
 }
+*/
 
 void	short_swap(p,n)
 unsigned short  *p;
@@ -342,7 +345,7 @@ char	*which_facility(int serial_number)
 			break;	
 		case 904:
 		case 911:
-		case 915:
+		case 916:
 			return("necat");
 			break;
 		case 907:
@@ -365,7 +368,7 @@ char	*which_facility(int serial_number)
 		case 909:
 			return("nsrrc");
 			break;
-		case 916:
+		case 915:
 			return("spring8");
 			break;
 		case 917:
@@ -378,11 +381,14 @@ char	*which_facility(int serial_number)
 		case 922:
 			return("dls");
 			break;
+		case 924:
+			return("fip");
+			break;
 		case 927:	
 			return("soleil");
 			break;
 		case 928:
-			return("asp");
+			return("as");
 			break;
 		case 929:
 			return("pohang");
@@ -421,7 +427,17 @@ void	smv_date_to_cbf_date(char *smv_date, char *cbf_date)
 
 }
 
-int	adscimg2cbf_sub(char *header, unsigned short *data, char *cbf_filename, int pack_flags)
+#define	BEAM_CENTER_FROM_HEADER	0
+#define BEAM_CENTER_MOSFLM	1
+#define	BEAM_CENTER_ULHC	2
+#define BEAM_CENTER_LLHC	3
+
+int	adscimg2cbf_sub(char *header, 
+                    unsigned short *data, 
+                    char *cbf_filename, 
+                    int pack_flags, 
+                    int beam_center_convention,
+                    int pad_flag)
 {
   FILE *out;
 
@@ -449,12 +465,14 @@ int	adscimg2cbf_sub(char *header, unsigned short *data, char *cbf_filename, int
   int		*ip;
   unsigned short *up;
   int		i, j;
+  char *  local_bo;
   int		this_bo, smv_bo;
   char		*header_as_details;
   int		header_size;
   double	oscillation_start, oscillation_range;
   double	detector_center_x, detector_center_y;
   double	det_beam_center_to_origin_dist_x, det_beam_center_to_origin_dist_y;
+  double	header_beam_x, header_beam_y;
 
 
 	if(0 == pack_flags)
@@ -673,22 +691,54 @@ int	adscimg2cbf_sub(char *header, unsigned short *data, char *cbf_filename, int
  *
  *	Define the origin to be the upper left hand corner, since
  *	this is the "storage origin" of the data array.
+ *
+ *	The incoming beam center value is nominally defined as the
+ *	adxv mm coordinate system:  origin in mm in the lower left hand
+ *	corner.
+ *
+ *	Other conventions are recognized either through additional header
+ *	keywords or "BEAM_CENTER" variants (future) found in the ADSC
+ *	header, or forced through the beam_center_convention parameter
+ *	to adscimg2cbf_sub() function.
  */
 
   s[0] = '\0';
   gethd("BEAM_CENTER_X", s, header);
   if('\0' == s[0])
-	det_beam_center_to_origin_dist_x = - detector_center_x;
+	header_beam_x = detector_center_x;
   else
-	det_beam_center_to_origin_dist_x = - atof(s);
+	header_beam_x = atof(s);
+
+  det_beam_center_to_origin_dist_x = - header_beam_x;
     
   s[0] = '\0';
   gethd("BEAM_CENTER_Y", s, header);
   if('\0' == s[0])
-	det_beam_center_to_origin_dist_y = detector_center_y;
+	header_beam_y = detector_center_y;
   else
-	det_beam_center_to_origin_dist_y = (smv_size1 - 1.5) * pixel_size - atof(s);
+	header_beam_y = atof(s);
     
+	switch(beam_center_convention)
+	{
+		case BEAM_CENTER_FROM_HEADER:
+		case BEAM_CENTER_LLHC:
+		default:
+
+			det_beam_center_to_origin_dist_x = - header_beam_x;
+			det_beam_center_to_origin_dist_y =   (smv_size1 - 1.5) * pixel_size - header_beam_y;
+			break;
+
+		case BEAM_CENTER_ULHC:
+			det_beam_center_to_origin_dist_x = - header_beam_x;
+			det_beam_center_to_origin_dist_y =   header_beam_y;
+			break;
+
+		case BEAM_CENTER_MOSFLM:
+			det_beam_center_to_origin_dist_x = - header_beam_y;
+			det_beam_center_to_origin_dist_y =   header_beam_x;
+			break;
+	}
+
     /* Date */
 
   s[0] = '\0';
@@ -769,7 +819,9 @@ int	adscimg2cbf_sub(char *header, unsigned short *data, char *cbf_filename, int
 /* Make sure to swap bytes if there is a change in byte order
  * between the machine which made the SMV file and this machine */
 
-  this_bo = getbo();
+  /* this_bo = getbo();*/
+  cbf_get_local_integer_byte_order(&local_bo);
+  this_bo = (local_bo[0]=='l'||local_bo[0]=='L')?0:1;
   smv_bo = this_bo;
 
   s[0] = '\0';
@@ -1445,7 +1497,7 @@ int	adscimg2cbf_sub(char *header, unsigned short *data, char *cbf_filename, int
 		*ip++ = 0x0000ffff & *up++;
 
 /*
- int cbf_set_integerarray_wdims (cbf_handle    handle,
+ int cbf_set_integerarray_wdims_fs (cbf_handle    handle,
                           unsigned int  compression,
                           int           id,
                           void         *value,
@@ -1453,12 +1505,12 @@ int	adscimg2cbf_sub(char *header, unsigned short *data, char *cbf_filename, int
                           int           elsign,
                           size_t        nelem,
                           const char   *byteorder,
-                          size_t        dim1,
-                          size_t        dim2,
-                          size_t        dim3,
+                          size_t        dimfast,
+                          size_t        dimmid,
+                          size_t        dimslow,
                           size_t        padding)
 */
- cbf_failnez( cbf_set_integerarray_wdims ((cbf_handle)    cbf,
+ cbf_failnez( cbf_set_integerarray_wdims_fs ((cbf_handle)    cbf,
                           (unsigned int) pack_flags,
                           (int)          1,
                                          data_as_int,
@@ -1467,7 +1519,7 @@ int	adscimg2cbf_sub(char *header, unsigned short *data, char *cbf_filename, int
                           (size_t)        smv_size1 * smv_size2,
                           	 	"little_endian",
                           (size_t)        smv_size1,
-                          (size_t)        smv_size1,
+                          (size_t)        smv_size2,
                           (size_t)        0,
                           (size_t)        0))
 
@@ -1482,7 +1534,7 @@ int	adscimg2cbf_sub(char *header, unsigned short *data, char *cbf_filename, int
     exit (1);
   }
 
-  cbf_failnez (cbf_write_file (cbf, out, 1, CBF, MSG_DIGEST | MIME_HEADERS, 0))
+  cbf_failnez (cbf_write_file (cbf, out, 1, CBF, MSG_DIGEST | MIME_HEADERS | pad_flag, 0))
   
 
     /* Free the cbf */
