@@ -1,7 +1,7 @@
 /**********************************************************************
  * cbf.h -- cbflib basic API functions                                *
  *                                                                    *
- * Version 0.7.9 30 December 2007                                     *
+ * Version 0.8.0 20 July 2008                                         *
  *                                                                    *
  *                          Paul Ellis and                            *
  *         Herbert J. Bernstein (yaya@bernstein-plus-sons.com)        *
@@ -265,24 +265,72 @@ extern "C" {
 
   /* Currently the cbf library assumes a 32-bit or larger integer */
 
+#ifndef SWIG
+  /* Something wrong with the SWIG preprocessor makes it barf on this when used on a 64-bit OS! */
 #if UINT_MAX / 65535U < 65535U
 #error cbflib assumes int is at least 32 bits
 #endif
+#else
+    #define CBF_USE_LONG_LONG
+    typedef long long CBF_sll_type;
+    typedef unsigned long long CBF_ull_type;
+#endif
+   
+#if defined(CBF_DONT_USE_LONG_LONG) || defined(__cplusplus) || defined(__MINGW32__)
+#undef ULLONG_MAX
+#undef CBF_USE_LONG_LONG
+#endif
 
+#ifndef SWIG
+#if defined(ULLONG_MAX) && defined(LLONG_MAX)
+  #if ULLONG_MAX >= 18446744073709551615U
+    #define  CBF_USE_LONG_LONG
+    typedef long long CBF_sll_type;
+    typedef unsigned long long CBF_ull_type;
+  #else
+    #if UINT_MAX >= 4294967295U
+      typedef struct { unsigned int el0:32;unsigned int el1:32;} CBF_sll_type;
+      typedef struct { unsigned int el0:32;unsigned int el1:32;} CBF_ull_type;
+      #define CBF_SLL_INTS 2
+      #define CBF_ULL_INTS 2
+    #else
+      typedef struct { unsigned int el0:32;unsigned int el1:32;unsigned int el2:32;unsigned int el3:32;} CBF_sll_type;
+      typedef struct { unsigned int el0:32;unsigned int el1:32;unsigned int el2:32;unsigned int el3:32;} CBF_ull_type;
+      #define CBF_SLL_INTS 4
+      #define CBF_ULL_INTS 4
+    #endif
+  #endif
+#else
+  #if UINT_MAX >= 4294967295U
+    typedef struct { unsigned int el0:32;unsigned int el1:32;} CBF_sll_type;
+    typedef struct { unsigned int el0:32;unsigned int el1:32;} CBF_ull_type;
+    #define CBF_SLL_INTS 2
+    #define CBF_ULL_INTS 2
+  #else
+    typedef struct { unsigned int el0:32;unsigned int el1:32;unsigned int el2:32;unsigned int el3:32;} CBF_sll_type;
+    typedef struct { unsigned int el0:32;unsigned int el1:32;unsigned int el2:32;unsigned int el3:32;} CBF_ull_type;
+    #define CBF_SLL_INTS 4
+    #define CBF_ULL_INTS 4
+  #endif
+#endif
+#endif
 
   /* API version and assumed dictionary version */
   
-#define CBF_API_VERSION     "CBFlib v0.7.8"
-#define CBF_DIC_VERSION     "CBF: VERSION 1.5"
+#define CBF_API_VERSION     "CBFlib v0.9.0"
+#define CBF_DIC_VERSION     "CBF: VERSION 1.6"
 
   /* Maximum line length */
 
 #define CBF_LINELENGTH_10 80
 #define CBF_LINELENGTH_11 2048
+#define CBF_LINELENGTH_20 8192
 
-  /* Initial write buffer size */
+  /* Initial io buffer sizes */
   
+#define CBF_INIT_READ_BUFFER 4096
 #define CBF_INIT_WRITE_BUFFER 4096
+#define CBF_TRANSFER_BUFFER 4096
 
 
   /* Error codes */
@@ -318,6 +366,26 @@ extern "C" {
 #define CBF_TOKEN_BIN        '\304'     /* Binary section              */
 #define CBF_TOKEN_MIME_BIN   '\305'     /* Mime-encoded binary section */
 #define CBF_TOKEN_TMP_BIN    '\306'     /* Temporary binary section    */
+#define CBF_TOKEN_BKTSTRING  '\311'     /* Composite string []         */
+#define CBF_TOKEN_BRCSTRING  '\312'     /* Composite string {}         */
+#define CBF_TOKEN_PRNSTRING  '\313'     /* Composite string ()         */
+#define CBF_TOKEN_TDQSTRING  '\314'     /* Triple Double-Quoted String */
+#define CBF_TOKEN_TSQSTRING  '\315'     /* Triple Single-Quoted String */
+#define CBF_TOKEN_BKTITEM    '\316'     /* Bracketed item              */
+#define CBF_TOKEN_FUNCTION   '\317'     /* Function definition         */
+
+#define cbf_token_term(tokentype) \
+  (((tokentype)==CBF_TOKEN_WORD)?' ':                \
+  (((tokentype)==CBF_TOKEN_SQSTRING)?'\'':           \
+  (((tokentype)==CBF_TOKEN_DQSTRING)?'"':            \
+  (((tokentype)==CBF_TOKEN_SCSTRING)?';':            \
+  (((tokentype)==CBF_TOKEN_BKTSTRING)?']':           \
+  (((tokentype)==CBF_TOKEN_BRCSTRING)?'}':           \
+  (((tokentype)==CBF_TOKEN_PRNSTRING)?')':           \
+  (((tokentype)==CBF_TOKEN_TDQSTRING)?'"':           \
+  (((tokentype)==CBF_TOKEN_TDQSTRING)?'\'': '\0' )))))))) )
+
+
 
   /* Constants for case sensitivity */
   
@@ -366,11 +434,35 @@ extern "C" {
 #define MSG_NODIGEST    0x0004  /* Do not check message digests       */
 #define MSG_DIGEST      0x0008  /* Check message digests              */
 #define MSG_DIGESTNOW   0x0010  /* Check message digests immediately  */
+#define MSG_DIGESTWARN  0x0020  /* Warn on message digests immediately*/
 #define PAD_1K          0x0020  /* Pad binaries with 1023 0's         */
 #define PAD_2K          0x0040  /* Pad binaries with 2047 0's         */
 #define PAD_4K          0x0080  /* Pad binaries with 4095 0's         */
 
 
+  /* Constants used to control CIF parsing */
+  
+#define CBF_PARSE_BRC   0x0100  /* PARSE DDLm/CIF2 brace {,...}             */
+#define CBF_PARSE_PRN   0x0200  /* PARSE DDLm parens     (,...)             */
+#define CBF_PARSE_BKT   0x0400  /* PARSE DDLm brackets   [,...]             */
+#define CBF_PARSE_BRACKETS \
+                        0x0700  /* PARSE ALL brackets                       */
+#define CBF_PARSE_TQ    0x0800  /* PARSE treble quotes """...""" and '''...'''       */
+#define CBF_PARSE_CIF2_DELIMS  \
+                        0x1000  /* Do not scan past an unescaped close quote
+                                   do not accept {} , : " ' in non-delimited
+                                   strings'{ */                          
+#define CBF_PARSE_DDLm  0x0700  /* For DDLm parse (), [], {}                */
+#define CBF_PARSE_CIF2  0x1F00  /* For CIF2 parse {}, treble quotes,
+                                   stop on unescaped close quotes           */
+#define CBF_PARSE_DEFINES      \
+                        0x2000  /* Recognize DEFINE_name            */      
+                        
+  
+#define CBF_PARSE_WIDE      0x4000  /* PARSE wide files                         */
+#define CBF_PARSE_WS        0x8000  /* PARSE whitespace                         */
+#define CBF_PARSE_UTF8      0x10000 /* PARSE UTF-8                              */
+
 #define HDR_DEFAULT (MIME_HEADERS | MSG_NODIGEST)
 
 #define MIME_NOHEADERS  PLAIN_HEADERS
@@ -460,7 +552,9 @@ typedef struct _cbf_handle_struct
   struct _cbf_handle_struct *dictionary;
   
   cbf_file * file;                   /* NULL or an active cbf_file for input */
-  
+      
+  cbf_file * commentfile;            /* NULL or file for whitespace and comments */
+
   int  startcolumn, startline;       /* starting location of last token */
   
   FILE * logfile;                    /* NULL or an active stream for error logging */
@@ -488,13 +582,20 @@ int cbf_free_handle (cbf_handle handle);
 
   /* Read a file */
 
-int cbf_read_file (cbf_handle handle, FILE *stream, int headers);
+int cbf_read_file (cbf_handle handle, FILE *stream, int flags);
 
 
 
   /* Read a wide file */
 
-int cbf_read_widefile (cbf_handle handle, FILE *stream, int headers);
+int cbf_read_widefile (cbf_handle handle, FILE *stream, int flags);
+
+
+  /* Read a pre-read buffered file */
+  
+int cbf_read_buffered_file (cbf_handle handle, FILE *stream, int flags, 
+                            const char * buffer, size_t buffer_len);
+
 
 
   /* Write a file */
@@ -798,6 +899,11 @@ int cbf_category_name (cbf_handle handle, const char **categoryname);
 int cbf_column_name (cbf_handle handle, const char **columnname);
 
 
+  /* Set the name of the current column */
+    
+int cbf_set_column_name (cbf_handle handle, const char *columnname);
+
+
   /* Get the number of the current row */
   
 int cbf_row_number (cbf_handle handle, unsigned int *row);
@@ -917,7 +1023,16 @@ int cbf_get_arrayparameters_wdims (cbf_handle    handle,
 #define cbf_get_arrayparameters_wdims_sf(handle, compression, id, elsize, elsigned, elunsigned, nelem, minelem, maxelem, realarray, byteorder, dimslow, dimmid, dimfast, padding) \
         cbf_get_arrayparameters_wdims((handle),(compression),(id),(elsize),(elsigned),(elunsigned),(nelem),(minelem),(maxelem),(realarray),(byteorder),(dimfast),(dimmid),(dimslow), (padding))
 
-
+    /* Get the dimensions of the current (row, column) array entry
+     from the CBF tags */
+    
+    
+    int cbf_get_arraydimensions(cbf_handle handle,
+                                size_t * dimover,
+                                size_t * dimfast,
+                                size_t * dimmid,
+                                size_t * dimslow);
+        
 
   /* Get the parameters of the current (row, column) integer array entry */
   
@@ -1069,6 +1184,10 @@ void cbf_error (const char *message);
   
 void cbf_log (cbf_handle handle, const char *message, int logflags);
 
+  /* issue a log message for a cbf_file */
+  
+void cbf_flog (cbf_file * file, const char *message, int logflags);
+
 
   /* Find a datablock, creating it if necessary */
   
@@ -1272,8 +1391,29 @@ int cbf_mpint_rightshift_acc(unsigned int * acc, size_t acsize, int shift);
   /* Shift accumulator left */
 
 int cbf_mpint_leftshift_acc(unsigned int * acc, size_t acsize, int shift);
+    
+    /* get accumulator bit length */
+    
+int cbf_mpint_get_acc_bitlength(unsigned int * acc, size_t acsize, size_t * bitlength);
 
+  /* Check value of type validity */
+  
+int cbf_check_type_contents(const char *type, const char *value);
 
+  /* Regex Match function */
+
+int cbf_match(const char *string, char *pattern);
+
+  /* Interpreter for dREL method expression */
+
+int cbf_drel(cbf_handle handle, cbf_handle dict, 
+                               const char *mainitemname, 
+                               const char *datablock, 
+                               const char *expression);
+
+  /* Construct Functions dictionary */
+
+int cbf_construct_functions_dictionary(cbf_handle dict, const char *datablockname, const char *functionname);
 
 #ifdef __cplusplus
 
