@@ -1,7 +1,7 @@
 /**********************************************************************
  * cbf_compress.h                                                     *
  *                                                                    *
- * Version 0.7.9 30 December 2007                                     *
+ * Version 0.8.0 20 July 2008                                         *
  *                                                                    *
  *                          Paul Ellis and                            *
  *         Herbert J. Bernstein (yaya@bernstein-plus-sons.com)        *
@@ -304,6 +304,7 @@ int cbf_decompress (void         *destination,
                     int           elsign, 
                     size_t        nelem, 
                     size_t       *nelem_read,
+                    size_t        compressedsize,
                     unsigned int  compression,
                     int           bits, 
                     int           sign,
@@ -315,10 +316,10 @@ int cbf_decompress (void         *destination,
                     size_t        dimmid,
                     size_t        dimslow,
                     size_t        padding);
-#define cbf_decompress_fs(destination,elsize,elsign,nelem,nelem_read,compression,bits,sign,file,realarray,byteorder,dimover,dimfast,dimmid,dimslow,padding) \
-        cbf_decompress((destination),(elsize),(elsign),(nelem),(nelem_read),(compression),(bits),(sign),(file),(realarray),(byteorder),(dimover),(dimfast),(dimmid),(dimslow),(padding)) 
-#define cbf_decompress_sf(destination,elsize,elsign,nelem,nelem_read,compression,bits,sign,file,realarray,byteorder,dimover,dimslow,dimmid,dimfast,padding) \
-        cbf_decompress((destination),(elsize),(elsign),(nelem),(nelem_read),(compression),(bits),(sign),(file),(realarray),(byteorder),(dimover),(dimfast),(dimmid),(dimslow),(padding)) 
+#define cbf_decompress_fs(destination,elsize,elsign,nelem,nelem_read,compressedsize,compression,bits,sign,file,realarray,byteorder,dimover,dimfast,dimmid,dimslow,padding) \
+        cbf_decompress((destination),(elsize),(elsign),(nelem),(nelem_read),(compressedsize),(compression),(bits),(sign),(file),(realarray),(byteorder),(dimover),(dimfast),(dimmid),(dimslow),(padding))
+#define cbf_decompress_sf(destination,elsize,elsign,nelem,nelem_read,compressedsize,compression,bits,sign,file,realarray,byteorder,dimover,dimslow,dimmid,dimfast,padding) \
+        cbf_decompress((destination),(elsize),(elsign),(nelem),(nelem_read),(compressedsize),(compression),(bits),(sign),(file),(realarray),(byteorder),(dimover),(dimfast),(dimmid),(dimslow),(padding))
 
 
 #ifdef __cplusplus
