@@ -266,6 +266,9 @@ extern "C" {
 typedef struct
 {
   FILE        *stream;            /* File pointer                           */
+  FILE        *logfile;           /* NULL or an active stream for error logging */
+  int          errors;            /* logged error messages                  */
+  int          warnings;          /* logged warnings                        */
   unsigned int connections;       /* Number of pointers to this structure   */
   int          temporary;         /* Flag for temporary file (memres)       */
   int          bits [2];          /* Buffer for bitwise reads and writes    */
@@ -318,7 +321,12 @@ int cbf_delete_fileconnection (cbf_file **file);
 int cbf_file_connections (cbf_file *file);
 
 
-  /* Set the size of the buffer */
+  /* Set the size of an input/output buffer */
+
+int cbf_set_io_buffersize (cbf_file *file, size_t size);
+
+
+  /* Set the size of an output buffer */
 
 int cbf_set_buffersize (cbf_file *file, size_t size);
 
@@ -333,6 +341,15 @@ int cbf_reset_buffer (cbf_file *file);
 int cbf_save_character (cbf_file *file, int c);
 
 
+  /* Add a character to the buffer, trimming trailing spaces */
+
+int cbf_save_character_trim (cbf_file *file, int c);
+
+    /* Add a character to the buffer at a given position */
+    
+int cbf_save_character_at (cbf_file *file, int c, size_t position);
+    
+
   /* Retrieve the buffer */
 
 int cbf_get_buffer (cbf_file *file, const char **buffer, 
@@ -391,6 +408,11 @@ int cbf_end_digest (cbf_file *file, char *digest);
 int cbf_reset_bits (cbf_file *file);
 
 
+  /* Discard any bits in the input buffers */
+
+int cbf_reset_in_bits (cbf_file *file);
+
+
   /* Discard any characters in the character buffers */
 
 int cbf_reset_characters (cbf_file *file);
