@@ -264,6 +264,9 @@ extern "C" {
 size_t memory_allocated;
 #endif
 
+#ifdef DMALLOC
+#include <dmalloc.h>
+#endif
 
   /* Reallocate a block of memory (never lose the old block on failure) */
 
@@ -363,7 +366,7 @@ int cbf_realloc (void **old_block, size_t *old_nelem,
 #endif
 
     free (*old_block);
-
+    
   }
 
 
