@@ -1,7 +1,7 @@
 /**********************************************************************
  * cbf_compress -- compression and decompression                      *
  *                                                                    *
- * Version 0.7.9 30 December 2007                                     *
+ * Version 0.8.0 20 July 2008                                         *
  *                                                                    *
  *                          Paul Ellis and                            *
  *         Herbert J. Bernstein (yaya@bernstein-plus-sons.com)        *
@@ -543,6 +543,7 @@ int cbf_decompress (void         *destination,
                     int           elsign,
                     size_t        nelem,
                     size_t       *nelem_read,
+                    size_t        compressedsize,
                     unsigned int  compression,
                     int           bits,
                     int           sign,
@@ -560,7 +561,7 @@ int cbf_decompress (void         *destination,
     case CBF_CANONICAL:
 
       return cbf_decompress_canonical (destination, elsize, elsign, nelem,
-                                       nelem_read, compression,
+                                       nelem_read, compressedsize, compression,
                                        bits, sign, file, realarray, byteorder,
                                        dimover, dimfast, dimmid, dimslow, padding);
 
@@ -569,28 +570,28 @@ int cbf_decompress (void         *destination,
     case 0:
 
       return cbf_decompress_packed (destination, elsize, elsign, nelem,
-                                    nelem_read, compression,
+                                    nelem_read, compressedsize, compression,
                                     bits, sign, file, realarray, byteorder,
                                     dimover, dimfast, dimmid, dimslow, padding);
 
     case CBF_BYTE_OFFSET:
 
       return cbf_decompress_byte_offset (destination, elsize, elsign, nelem,
-                                         nelem_read, compression,
+                                         nelem_read, compressedsize, compression,
                                          bits, sign, file, realarray, byteorder,
                                          dimover, dimfast, dimmid, dimslow, padding);
 
     case CBF_PREDICTOR:
 
       return cbf_decompress_predictor (destination, elsize, elsign, nelem,
-                                       nelem_read, compression,
+                                       nelem_read, compressedsize, compression,
                                        bits, sign, file, realarray, byteorder,
                                        dimover, dimfast, dimmid, dimslow, padding);
 
     case CBF_NONE:
 
       return cbf_decompress_none (destination, elsize, elsign, nelem,
-                                  nelem_read, compression,
+                                  nelem_read, compressedsize, compression,
                                   bits, sign, file, realarray, byteorder,
                                   dimover, dimfast, dimmid, dimslow, padding);
   }
