@@ -1,7 +1,7 @@
 /**********************************************************************
  * cbf_uncompressed -- uncompressed binary sections                   *
  *                                                                    *
- * Version 0.7.9   30 December 2007                                   *
+ * Version 0.8.0 20 July 2008                                         *
  *                                                                    *
  *                          Paul Ellis and                            *
  *         Herbert J. Bernstein (yaya@bernstein-plus-sons.com)        *
@@ -591,6 +591,7 @@ int cbf_decompress_none (void         *destination,
                          int           elsign,
                          size_t        nelem,
                          size_t       *nelem_read,
+                         size_t        compressedsize,
                          unsigned int  compression,
                          int           data_bits,
                          int           data_sign,
